@@ -1,6 +1,10 @@
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
+
 """ The GUI """
+
 import glob
 import os
 import signal
@@ -12,9 +16,9 @@ from threading import Lock, RLock
 from PyQt5.Qt import (
     QT_VERSION, QApplication, QBuffer, QByteArray, QCoreApplication, QDateTime,
     QDesktopServices, QDialog, QEvent, QFileDialog, QFileIconProvider, QFileInfo,
-    QFont, QFontDatabase, QFontInfo, QFontMetrics, QIcon, QLocale, QObject,
-    QSettings, QSocketNotifier, QStringListModel, Qt, QThread, QTimer, QTranslator,
-    QUrl, pyqtSignal
+    QFont, QFontDatabase, QFontInfo, QFontMetrics, QIcon, QLocale,
+    QNetworkProxyFactory, QObject, QSettings, QSocketNotifier, QStringListModel, Qt,
+    QThread, QTimer, QTranslator, QUrl, pyqtSignal
 )
 from PyQt5.QtWidgets import QStyle  # Gives a nicer error message than import from Qt
 
@@ -34,9 +38,10 @@ from calibre.utils.config import Config, ConfigProxy, JSONConfig, dynamic
 from calibre.utils.date import UNDEFINED_DATE
 from calibre.utils.file_type_icons import EXT_MAP
 from calibre.utils.localization import get_lang
-from polyglot.builtins import (iteritems, itervalues, unicode_type,
-        string_or_bytes, range, map)
 from polyglot import queue
+from polyglot.builtins import (
+    iteritems, itervalues, range, string_or_bytes, unicode_type, map
+)
 
 try:
     NO_URL_FORMATTING = QUrl.None_
@@ -354,7 +359,7 @@ def is_widescreen():
     global _is_widescreen
     if _is_widescreen is None:
         try:
-            _is_widescreen = float(available_width())/available_height() > 1.4
+            _is_widescreen = available_width()/available_height() > 1.4
         except:
             _is_widescreen = False
     return _is_widescreen
@@ -687,7 +692,7 @@ def pixmap_to_data(pixmap, format='JPEG', quality=None):
     buf = QBuffer(ba)
     buf.open(QBuffer.WriteOnly)
     pixmap.save(buf, format, quality=quality)
-    return bytes(ba.data())
+    return ba.data()
 
 
 def decouple(prefix):
@@ -736,28 +741,28 @@ class Translator(QTranslator):
         try:
             src = unicode_type(args[1])
         except:
-            return u''
+            return ''
         t = _
         return t(src)
 
 
 gui_thread = None
-
 qt_app = None
 
-builtin_fonts_loaded = False
+
+def calibre_font_files():
+    return glob.glob(P('fonts/liberation/*.?tf')) + [P('fonts/calibreSymbols.otf')] + \
+            glob.glob(os.path.join(config_dir, 'fonts', '*.?tf'))
 
 
 def load_builtin_fonts():
     global _rating_font, builtin_fonts_loaded
     # Load the builtin fonts and any fonts added to calibre by the user to
     # Qt
-    if builtin_fonts_loaded:
+    if hasattr(load_builtin_fonts, 'done'):
         return
-    builtin_fonts_loaded = True
-    for ff in glob.glob(P('fonts/liberation/*.?tf')) + \
-            [P('fonts/calibreSymbols.otf')] + \
-            glob.glob(os.path.join(config_dir, 'fonts', '*.?tf')):
+    load_builtin_fonts.done = True
+    for ff in calibre_font_files():
         if ff.rpartition('.')[-1].lower() in {'ttf', 'otf'}:
             with open(ff, 'rb') as s:
                 # Windows requires font files to be executable for them to be
@@ -766,8 +771,8 @@ def load_builtin_fonts():
                 if fid > -1:
                     fam = QFontDatabase.applicationFontFamilies(fid)
                     fam = set(map(unicode_type, fam))
-                    if u'calibre Symbols' in fam:
-                        _rating_font = u'calibre Symbols'
+                    if 'calibre Symbols' in fam:
+                        _rating_font = 'calibre Symbols'
 
 
 def setup_gui_option_parser(parser):
@@ -808,11 +813,40 @@ def setup_hidpi():
         prints('Not controlling automatic hidpi scaling')
 
 
+def setup_unix_signals(self):
+    if hasattr(os, 'pipe2'):
+        read_fd, write_fd = os.pipe2(os.O_CLOEXEC | os.O_NONBLOCK)
+    else:
+        import fcntl
+        read_fd, write_fd = os.pipe()
+        cloexec_flag = getattr(fcntl, 'FD_CLOEXEC', 1)
+        for fd in (read_fd, write_fd):
+            flags = fcntl.fcntl(fd, fcntl.F_GETFD)
+            fcntl.fcntl(fd, fcntl.F_SETFD, flags | cloexec_flag | os.O_NONBLOCK)
+
+    original_handlers = {}
+    for sig in (signal.SIGINT, signal.SIGTERM):
+        original_handlers[sig] = signal.signal(sig, lambda x, y: None)
+        signal.siginterrupt(sig, False)
+    signal.set_wakeup_fd(write_fd)
+    self.signal_notifier = QSocketNotifier(read_fd, QSocketNotifier.Read, self)
+    self.signal_notifier.setEnabled(True)
+    self.signal_notifier.activated.connect(self.signal_received, type=Qt.QueuedConnection)
+    return original_handlers
+
+
 class Application(QApplication):
 
     shutdown_signal_received = pyqtSignal()
 
-    def __init__(self, args, force_calibre_style=False, override_program_name=None, headless=False, color_prefs=gprefs):
+    def __init__(self, args, force_calibre_style=False, override_program_name=None, headless=False, color_prefs=gprefs, windows_app_uid=None):
+        QNetworkProxyFactory.setUseSystemConfiguration(True)
+        if iswindows:
+            self.windows_app_uid = None
+            if windows_app_uid:
+                windows_app_uid = unicode_type(windows_app_uid)
+                if set_app_uid(windows_app_uid):
+                    self.windows_app_uid = windows_app_uid
         self.file_event_hook = None
         if isfrozen and QT_VERSION <= 0x050700 and 'wayland' in os.environ.get('QT_QPA_PLATFORM', ''):
             os.environ['QT_QPA_PLATFORM'] = 'xcb'
@@ -836,7 +870,10 @@ class Application(QApplication):
         QApplication.setApplicationName(APP_UID)
         if override_program_name and hasattr(QApplication, 'setDesktopFileName'):
             QApplication.setDesktopFileName(override_program_name)
+        QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)  # needed for webengine
         QApplication.__init__(self, qargs)
+        if isosx:
+            plugins['cocoa'][0].disable_cocoa_ui_elements()
         self.setAttribute(Qt.AA_UseHighDpiPixmaps)
         self.setAttribute(Qt.AA_SynthesizeTouchForUnhandledMouseEvents, False)
         try:
@@ -875,7 +912,7 @@ class Application(QApplication):
         self.line_height = max(12, QFontMetrics(self.font()).lineSpacing())
 
         dl = QLocale(get_lang())
-        if unicode_type(dl.bcp47Name()) != u'C':
+        if unicode_type(dl.bcp47Name()) != 'C':
             QLocale.setDefault(dl)
         global gui_thread, qt_app
         gui_thread = QThread.currentThread()
@@ -897,7 +934,7 @@ class Application(QApplication):
                 'calibre versions newer than 2.0 do not run on Windows XP. This is'
                 ' because the graphics toolkit calibre uses (Qt 5) crashes a lot'
                 ' on Windows XP. We suggest you stay with <a href="%s">calibre 1.48</a>'
-                ' which works well on Windows XP.') % 'http://download.calibre-ebook.com/1.48.0/', show=True)
+                ' which works well on Windows XP.') % 'https://download.calibre-ebook.com/1.48.0/', show=True)
             raise SystemExit(1)
 
         if iswindows:
@@ -972,22 +1009,22 @@ class Application(QApplication):
         icon_map = self.__icon_map_memory_ = {}
         pcache = {}
         for k, v in iteritems({
-            'DialogYesButton': u'ok.png',
-            'DialogNoButton': u'window-close.png',
-            'DialogCloseButton': u'window-close.png',
-            'DialogOkButton': u'ok.png',
-            'DialogCancelButton': u'window-close.png',
-            'DialogHelpButton': u'help.png',
-            'DialogOpenButton': u'document_open.png',
-            'DialogSaveButton': u'save.png',
-            'DialogApplyButton': u'ok.png',
-            'DialogDiscardButton': u'trash.png',
-            'MessageBoxInformation': u'dialog_information.png',
-            'MessageBoxWarning': u'dialog_warning.png',
-            'MessageBoxCritical': u'dialog_error.png',
-            'MessageBoxQuestion': u'dialog_question.png',
-            'BrowserReload': u'view-refresh.png',
-            'LineEditClearButton': u'clear_left.png',
+            'DialogYesButton': 'ok.png',
+            'DialogNoButton': 'window-close.png',
+            'DialogCloseButton': 'window-close.png',
+            'DialogOkButton': 'ok.png',
+            'DialogCancelButton': 'window-close.png',
+            'DialogHelpButton': 'help.png',
+            'DialogOpenButton': 'document_open.png',
+            'DialogSaveButton': 'save.png',
+            'DialogApplyButton': 'ok.png',
+            'DialogDiscardButton': 'trash.png',
+            'MessageBoxInformation': 'dialog_information.png',
+            'MessageBoxWarning': 'dialog_warning.png',
+            'MessageBoxCritical': 'dialog_error.png',
+            'MessageBoxQuestion': 'dialog_question.png',
+            'BrowserReload': 'view-refresh.png',
+            'LineEditClearButton': 'clear_left.png',
         }):
             if v not in pcache:
                 p = I(v)
@@ -1054,23 +1091,7 @@ class Application(QApplication):
         self.setQuitOnLastWindowClosed(True)
 
     def setup_unix_signals(self):
-        import fcntl
-        if hasattr(os, 'pipe2'):
-            read_fd, write_fd = os.pipe2(os.O_CLOEXEC | os.O_NONBLOCK)
-        else:
-            read_fd, write_fd = os.pipe()
-            cloexec_flag = getattr(fcntl, 'FD_CLOEXEC', 1)
-            for fd in (read_fd, write_fd):
-                flags = fcntl.fcntl(fd, fcntl.F_GETFD)
-                fcntl.fcntl(fd, fcntl.F_SETFD, flags | cloexec_flag | os.O_NONBLOCK)
-
-        for sig in (signal.SIGINT, signal.SIGTERM):
-            signal.signal(sig, lambda x, y: None)
-            signal.siginterrupt(sig, False)
-        signal.set_wakeup_fd(write_fd)
-        self.signal_notifier = QSocketNotifier(read_fd, QSocketNotifier.Read, self)
-        self.signal_notifier.setEnabled(True)
-        self.signal_notifier.activated.connect(self.signal_received, type=Qt.QueuedConnection)
+        setup_unix_signals(self)
 
     def signal_received(self, read_fd):
         try:
@@ -1089,12 +1110,12 @@ def sanitize_env_vars():
     is needed to prevent library conflicts when launching external utilities.'''
 
     if islinux and isfrozen:
-        env_vars = {'LD_LIBRARY_PATH':'/lib', 'QT_PLUGIN_PATH':'/lib/qt_plugins'}
+        env_vars = {'LD_LIBRARY_PATH':'/lib'}
     elif iswindows:
-        env_vars = {k:None for k in 'QT_PLUGIN_PATH'.split()}
+        env_vars = {}
     elif isosx:
         env_vars = {k:None for k in (
-                    'FONTCONFIG_FILE FONTCONFIG_PATH QT_PLUGIN_PATH SSL_CERT_FILE').split()}
+                    'FONTCONFIG_FILE FONTCONFIG_PATH SSL_CERT_FILE').split()}
     else:
         env_vars = {}
 
@@ -1134,6 +1155,18 @@ def open_url(qurl):
         QDesktopServices.openUrl(qurl)
 
 
+def safe_open_url(qurl):
+    if isinstance(qurl, string_or_bytes):
+        qurl = QUrl(qurl)
+    if qurl.scheme() in ('', 'file'):
+        path = qurl.toLocalFile()
+        ext = os.path.splitext(path)[-1].lower()[1:]
+        if ext in ('exe', 'com', 'cmd', 'bat', 'sh', 'psh', 'ps1', 'vbs', 'js', 'wsf', 'vba', 'py', 'rb', 'pl', 'app'):
+            prints('Refusing to open file:', path)
+            return
+    open_url(qurl)
+
+
 def get_current_db():
     '''
     This method will try to return the current database in use by the user as
@@ -1168,6 +1201,10 @@ def ensure_app(headless=True):
             has_headless = isosx or islinux or isbsd
             if headless and has_headless:
                 args += ['-platformpluginpath', plugins_loc, '-platform', 'headless']
+                if isosx:
+                    os.environ['QT_MAC_DISABLE_FOREGROUND_APPLICATION_TRANSFORM'] = '1'
+            if headless and iswindows:
+                QApplication.setAttribute(Qt.AA_UseSoftwareOpenGL, True)
             _store_app = QApplication(args)
             if headless and has_headless:
                 _store_app.headless = True
@@ -1187,6 +1224,12 @@ def ensure_app(headless=True):
                 except:
                     pass
             sys.excepthook = eh
+    return _store_app
+
+
+def destroy_app():
+    global _store_app
+    _store_app = None
 
 
 def app_is_headless():
@@ -1236,7 +1279,7 @@ def elided_text(text, font=None, width=300, pos='middle'):
     from PyQt5.Qt import QFontMetrics, QApplication
     fm = QApplication.fontMetrics() if font is None else (font if isinstance(font, QFontMetrics) else QFontMetrics(font))
     delta = 4
-    ellipsis = u'\u2026'
+    ellipsis = '\u2026'
 
     def remove_middle(x):
         mid = len(x) // 2
@@ -1322,46 +1365,10 @@ def event_type_name(ev_or_etype):
     return 'UnknownEventType'
 
 
-def secure_web_page(qwebpage_or_qwebsettings):
-    from PyQt5.QtWebKit import QWebSettings
-    settings = qwebpage_or_qwebsettings if isinstance(qwebpage_or_qwebsettings, QWebSettings) else qwebpage_or_qwebsettings.settings()
-    settings.setAttribute(QWebSettings.JavaEnabled, False)
-    settings.setAttribute(QWebSettings.PluginsEnabled, False)
-    settings.setAttribute(QWebSettings.JavascriptCanOpenWindows, False)
-    settings.setAttribute(QWebSettings.JavascriptCanAccessClipboard, False)
-    settings.setAttribute(QWebSettings.LocalContentCanAccessFileUrls, False)  # ensure javascript cannot read from local files
-    settings.setAttribute(QWebSettings.NotificationsEnabled, False)
-    settings.setThirdPartyCookiePolicy(QWebSettings.AlwaysBlockThirdPartyCookies)
-    settings.setAttribute(QWebSettings.OfflineStorageDatabaseEnabled, False)
-    settings.setAttribute(QWebSettings.LocalStorageEnabled, False)
-    QWebSettings.setOfflineStorageDefaultQuota(0)
-    QWebSettings.setOfflineStoragePath(None)
-    return settings
-
-
 empty_model = QStringListModel([''])
 empty_index = empty_model.index(0)
 
 
-def get_app_uid():
-    import ctypes
-    from ctypes import wintypes
-    lpBuffer = wintypes.LPWSTR()
-    try:
-        AppUserModelID = ctypes.windll.shell32.GetCurrentProcessExplicitAppUserModelID
-    except Exception:  # Vista has no app uids
-        return
-    AppUserModelID.argtypes = [wintypes.LPWSTR]
-    AppUserModelID.restype = wintypes.HRESULT
-    try:
-        AppUserModelID(ctypes.cast(ctypes.byref(lpBuffer), wintypes.LPWSTR))
-    except Exception:
-        return
-    appid = lpBuffer.value
-    ctypes.windll.ole32.CoTaskMemFree(lpBuffer)
-    return appid
-
-
 def set_app_uid(val):
     import ctypes
     from ctypes import wintypes
@@ -1374,11 +1381,11 @@ def set_app_uid(val):
     try:
         AppUserModelID(unicode_type(val))
     except Exception as err:
-        prints(u'Failed to set app uid with error:', as_unicode(err))
+        prints('Failed to set app uid with error:', as_unicode(err))
         return False
     return True
 
 
 def add_to_recent_docs(path):
-    app_id = get_app_uid()
-    plugins['winutil'][0].add_to_recent_docs(unicode_type(path), app_id)
+    app = QApplication.instance()
+    plugins['winutil'][0].add_to_recent_docs(unicode_type(path), app.windows_app_uid)
